/*
                 Copyright(c) 2005 Toufik Sarni

This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA*/


#include <kernel.h>
#include <memory.h>
#include <proc.h>

struct entete_file  file_pret_tpartage={NULL,NULL,0};

/******************************************************************/
/*      ORDONNANCEMENT DES PROCESSUS EN TEMPS PARTAGE             */
/******************************************************************/

TACHE processus_pred()      //trouve le processus prcdent (le processus interrompu)
 {                                 //on est dans la tache scheduler, car tout processus interrompu suite  une IT Horloge, commute vers la tache scheduler
   u16 task_reg=scheduler.tss.link;       //link est mis  jour par le processeur lors de l'IT
   if (task_reg==KR_TACHE)           //si l'IT vient de la tche Kernel (sans doute premire utilisation)
     {
      if (file_pret_tpartage.tete !=NULL)  //et si la file des processus prets n'est pas vide
         {
          struct tache *ptr=file_pret_tpartage.tete;
          scheduler.tss.link=ptr->tss.tr;            
          return ptr;
         }
     }                   // fin si du cas de la premire execution.

   return (struct tache *)get_tss_adr(task_reg);
 }

TACHE round_robin(struct tache *tache)        //renvoie le prochain processus  partir de la file des processus prets
   {
    if (file_pret_tpartage.tete==NULL)
       return NULL;

    if (tache->sched.succ==NULL)
       return file_pret_tpartage.tete;       
    else
       return tache->sched.succ;
        
   }
void dispatch(struct tache *succ)
   {  

    scheduler.tss.link=succ->tss.tr;

    u32 *gdt=(u32 *)(adr_init_gdt+SCHED_TACHE); // <=> le pointeur gdt=0x40040
    *(++gdt) &=0xfffffdff;               // mise  1 du bit Busy dans le descripteur tss de la tche Scheduler

    gdt=(u32 *)(adr_init_gdt+(succ->tss.tr));
    *(++gdt) |=0x200;                 // mise  1 du bit Busy dans le descripteur tss du succsseur

    }

void schedule()             //routine execute  chaque top d'horloge (TIMER)
  {
   TACHE succ= round_robin(processus_pred());
      dispatch(succ);          //ralise la commutation
  }


/*****************************************************************************************/
/*****************************************************************************************/

struct tache * get_tache(int n, struct entete_file *entete)
 {
  int k;
   struct tache *ptr=entete->tete;
   for (k=2;k<=n;k++)
    {
      ptr=ptr->sched.succ;
    }
  return ptr;
 }

int add_file_pret(struct tache *tache,struct entete_file *entete)    //ajouter une tche dans une file
 {
   struct tache *ptr;

  if (entete->tete==NULL)        //si c'est la 1re "tche"  inserer
    {
    entete->tete=tache;
    entete->queue=tache;
    entete->cpt++;
    tache->sched.succ=NULL;
    tache->sched.pred=NULL;
    return 0;
    };
//sinon
   ptr=entete->queue;
   ptr->sched.succ=tache;

   tache->sched.pred=ptr;
   tache->sched.succ=NULL;                     //indiquer le dernier de la liste

   entete->queue=tache;          //la queue va pointer sur le deriner lment
   entete->cpt++;
 }

int rem_file_pret(struct tache *tache,struct entete_file *entete)                //supprimer de la liste, un processus
 {
    struct tache *ptr;
    if (entete->tete==NULL) return -1;       //erreur: aucun lment  supprimer

    if (((tache->sched.pred)==NULL)                  //si c'est le seul dans la file
       && ((tache->sched.succ)==NULL)) 
       {
       entete->tete=NULL;
       entete->queue=NULL; entete->cpt=0;
       return 0;
       }

    ptr=(struct tache *)tache->sched.pred;

    if (ptr!=NULL)                    //si ce n'est pas le premier dans la file
    {
    ptr->sched.succ=tache->sched.succ;

    ptr=tache->sched.succ;
      if (ptr!=NULL)                 //si ce n'est pas le dernier de la file
           ptr->sched.pred=tache->sched.pred;
      else entete->queue=tache->sched.pred;           //si c'est le dernier, mettre  jour la queue

     }

    else    // si c'est le premier
      {
         ptr=tache->sched.succ;
         ptr->sched.pred=NULL;
         entete->tete=tache->sched.succ;
         
      }

   entete->cpt--;
   return 0;
 }
